<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2019 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.1/1829459
 */
use Events\Listener\ListenerFactory as EventListenerFactory;
use Zend\Mvc\MvcEvent;
use Zend\Mvc\ResponseSender\SendResponseEvent;

$listeners = [
    Application\Listener\EventErrorListener::class,
    Application\Listener\RouteListener::class,
    Application\Listener\WorkerListener::class,
    Application\View\Http\StrictJsonStrategy::class,
    Application\View\Http\ExceptionStrategy::class,
    Application\Response\CallbackResponseSender::class,
    Application\Permissions\Csrf\Listener::class,
    Attachments\Listener\AttachmentsListener::class,
    Changes\Listener\CommitShelveListener::class,
    Changes\Listener\ChangeListener::class,
    Comments\Listener\CommentListener::class,
    Files\Listener\FileListener::class,
    Groups\Listener\GroupsListener::class,
    Jira\Listener\JiraListener::class,
    Jobs\Listener\JobListener::class,
    Mail\Listener\MailListener::class,
    Queue\Listener\Ping::class,
    Reviews\Listener\ShelveCommit::class,
    Reviews\Listener\Review::class,
    Users\Listener\UserEventListener::class,
    Users\Authentication\BasicAuthListener::class,
    Xhprof\Listener\XhprofListener::class,
    Application\Log\EventListener::class,
    Activity\Listener\ActivityListener::class
];

return [
    'listeners'                                 => $listeners,
    'service_manager'                           =>[
        'factories' => array_fill_keys(
            $listeners,
            EventListenerFactory::class
        )
    ],
    EventListenerFactory::EVENT_LISTENER_CONFIG => [
        EventListenerFactory::ALL                             => [
            Application\Log\EventListener::class => [
                [
                    // Ensure this is the first to fire with a high priority
                    EventListenerFactory::PRIORITY => 1000,
                    EventListenerFactory::CALLBACK => 'handleEventTriggered',
                ],
                [
                    // Ensure this is the last to fire with a low priority
                    EventListenerFactory::PRIORITY => -1000,
                    EventListenerFactory::CALLBACK => 'handleEventFinished',
                ]
            ]
        ],
        MvcEvent::EVENT_DISPATCH                              => [
            Application\Permissions\Csrf\Listener::class => [
                [
                    EventListenerFactory::PRIORITY => 100,
                    EventListenerFactory::CALLBACK => 'registerControllerListener'
                ]
            ],
            Users\Authentication\BasicAuthListener::class => [
                [
                    EventListenerFactory::PRIORITY => 100,
                    EventListenerFactory::CALLBACK => 'registerControllerListener'
                ]
            ]
        ],
        MvcEvent::EVENT_ROUTE                                 => [
            Application\Listener\RouteListener::class => [
                [
                    EventListenerFactory::PRIORITY => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK => 'handleMultiP4d'
                ],
                [
                    EventListenerFactory::PRIORITY => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK => 'handleApiUrl'
                ],
                [
                    EventListenerFactory::PRIORITY => -1000,
                    EventListenerFactory::CALLBACK => 'handleRequireLogin'
                ]
            ],
            Xhprof\Listener\XhprofListener::class => [
                [
                    EventListenerFactory::PRIORITY => -1010,
                    EventListenerFactory::CALLBACK => 'handleRouteEvent'
                ],
            ]
        ],
        MvcEvent::EVENT_RENDER                                => [
            Application\View\Http\StrictJsonStrategy::class => [
                [
                    EventListenerFactory::PRIORITY => -200,
                    EventListenerFactory::CALLBACK => 'injectStrictJsonResponse'
                ]
            ]
        ],
        MvcEvent::EVENT_DISPATCH_ERROR                        => [
            Application\Listener\EventErrorListener::class => [
                [
                    EventListenerFactory::PRIORITY => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK => 'onError'
                ]
            ],
            Application\View\Http\ExceptionStrategy::class => [
                [
                    EventListenerFactory::PRIORITY => 100,
                    EventListenerFactory::CALLBACK => 'prepareExceptionViewModel'
                ]
            ]
        ],
        MvcEvent::EVENT_RENDER_ERROR                          => [
            Application\Listener\EventErrorListener::class => [
                [
                    EventListenerFactory::PRIORITY => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK => 'onError'
                ]
            ],
            Application\View\Http\StrictJsonStrategy::class => [
                [
                    EventListenerFactory::PRIORITY => -200,
                    EventListenerFactory::CALLBACK => 'injectStrictJsonResponse'
                ]
            ]
        ],
        SendResponseEvent::EVENT_SEND_RESPONSE                => [
            Application\Response\CallbackResponseSender::class => [
                [
                    EventListenerFactory::PRIORITY        => -3500,
                    EventListenerFactory::CALLBACK        => '__invoke',
                    EventListenerFactory::MANAGER_CONTEXT => 'SendResponseListener'
                ]
            ]
        ],
        EventListenerFactory::WORKER_STARTUP                  => [
            Activity\Listener\ActivityListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'prePopulateActivity',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Application\Listener\WorkerListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'setHostUrl',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Jira\Listener\JiraListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'refreshProjectList',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
        ],
        EventListenerFactory::WORKER_SHUTDOWN                 => [
            Application\Listener\WorkerListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'removeInvalidatedFiles',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Xhprof\Listener\XhprofListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'handleWorkerShutdown',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ],
            ]
        ],
        EventListenerFactory::WORKER_LOOP                     => [
            Queue\Listener\Ping::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'sendPing',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ]
        ],
        EventListenerFactory::PING                            => [
            Queue\Listener\Ping::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'receivePing',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ]
        ],
        EventListenerFactory::CLEANUP_ATTACHMENTS             => [
            Attachments\Listener\AttachmentsListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'cleanUp',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ]
        ],
        EventListenerFactory::CLEANUP_ARCHIVE                 => [
            Files\Listener\FileListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'cleanUpArchive',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ]
        ],
        EventListenerFactory::COMMIT                          => [
            Changes\Listener\CommitShelveListener::class => [
                [
                    EventListenerFactory::PRIORITY        => 300,
                    EventListenerFactory::CALLBACK        => 'onCommitShelve',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ],
                [
                    EventListenerFactory::PRIORITY        => 200,
                    EventListenerFactory::CALLBACK        => 'activityAndMail',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ],
                [
                    EventListenerFactory::PRIORITY        => -100,
                    EventListenerFactory::CALLBACK        => 'postActivityAndMail',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ],
                [
                    EventListenerFactory::PRIORITY        => -200,
                    EventListenerFactory::CALLBACK        => 'configureMailReviewers',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ],
            ],
            Reviews\Listener\ShelveCommit::class => [
                [
                    EventListenerFactory::PRIORITY        => 100,
                    EventListenerFactory::CALLBACK        => 'lockThenProcess',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Activity\Listener\ActivityListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_ACTIVITY,
                    EventListenerFactory::CALLBACK        => 'createActivity',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Mail\Listener\MailListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_MAIL_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'handleMail',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Jira\Listener\JiraListener::class => [
                [
                    EventListenerFactory::PRIORITY        => -400,
                    EventListenerFactory::CALLBACK        => 'checkChange',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
        ],
        EventListenerFactory::SHELVE                          => [
            Changes\Listener\CommitShelveListener::class => [
                [
                    EventListenerFactory::PRIORITY        => 300,
                    EventListenerFactory::CALLBACK        => 'onCommitShelve',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Reviews\Listener\ShelveCommit::class => [
                [
                    EventListenerFactory::PRIORITY        => 200,
                    EventListenerFactory::CALLBACK        => 'processGitShelve',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ],
                [
                    EventListenerFactory::PRIORITY        => 100,
                    EventListenerFactory::CALLBACK        => 'lockThenProcess',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ],

            ],
            Activity\Listener\ActivityListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_ACTIVITY,
                    EventListenerFactory::CALLBACK        => 'createActivity',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Mail\Listener\MailListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_MAIL_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'handleMail',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
        ],
        EventListenerFactory::SHELVE_DELETE                   => [
            Reviews\Listener\ShelveCommit::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'shelveDelete',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Activity\Listener\ActivityListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_ACTIVITY,
                    EventListenerFactory::CALLBACK        => 'createActivity',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
        ],
        EventListenerFactory::CHANGE_SAVE                     => [
            Changes\Listener\ChangeListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'handleChangeSave',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ]
        ],
        EventListenerFactory::CHANGE_SAVED                    => [
            Changes\Listener\ChangeListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'descriptionSync',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ]
        ],
        EventListenerFactory::COMMENT                         => [
            Comments\Listener\CommentListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'commentCreated',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Activity\Listener\ActivityListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_ACTIVITY,
                    EventListenerFactory::CALLBACK        => 'createActivity',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Mail\Listener\MailListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_MAIL_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'handleMail',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
        ],
        EventListenerFactory::COMMENT_BATCH                   => [
            Comments\Listener\CommentListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'commentBatch',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Activity\Listener\ActivityListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_ACTIVITY,
                    EventListenerFactory::CALLBACK        => 'createActivity',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Mail\Listener\MailListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_MAIL_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'handleMail',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
        ],
        EventListenerFactory::COMMENT_SEND_DELAY              => [
            Comments\Listener\CommentListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'commentSendDelay',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Activity\Listener\ActivityListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_ACTIVITY,
                    EventListenerFactory::CALLBACK        => 'createActivity',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Mail\Listener\MailListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_MAIL_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'handleMail',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
        ],
        EventListenerFactory::GROUP                           => [
            Groups\Listener\GroupsListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'onGroup',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ]
        ],
        EventListenerFactory::GROUP_DELETE                    => [
            Groups\Listener\GroupsListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'onGroup',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ]
        ],
        EventListenerFactory::JOB                             => [
            Jobs\Listener\JobListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'handleJob',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Activity\Listener\ActivityListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_ACTIVITY,
                    EventListenerFactory::CALLBACK        => 'createActivity',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Mail\Listener\MailListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_MAIL_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'handleMail',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Jira\Listener\JiraListener::class => [
                [
                    EventListenerFactory::PRIORITY        => -300,
                    EventListenerFactory::CALLBACK        => 'handleJob',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
        ],
        EventListenerFactory::CHANGE                          => [
            Jira\Listener\JiraListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'checkChange',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ]
        ],
        EventListenerFactory::REVIEW                          => [
            Reviews\Listener\Review::class => [
                [
                    EventListenerFactory::PRIORITY        => 100,
                    EventListenerFactory::CALLBACK        => 'lockThenProcess',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Activity\Listener\ActivityListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_ACTIVITY,
                    EventListenerFactory::CALLBACK        => 'createActivity',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Mail\Listener\MailListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_MAIL_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'handleMail',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
            Jira\Listener\JiraListener::class => [
                [
                    EventListenerFactory::PRIORITY        => -300,
                    EventListenerFactory::CALLBACK        => 'checkReview',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
        ],
        EventListenerFactory::USER                            => [
            Users\Listener\UserEventListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'onUser',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ]
        ],
        EventListenerFactory::USER_DELETE                     => [
            Users\Listener\UserEventListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::DEFAULT_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'onUser',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ]
        ],
        EventListenerFactory::MAIL                            => [
            Mail\Listener\MailListener::class => [
                [
                    EventListenerFactory::PRIORITY        => EventListenerFactory::HANDLE_MAIL_PRIORITY,
                    EventListenerFactory::CALLBACK        => 'handleMail',
                    EventListenerFactory::MANAGER_CONTEXT => 'queue'
                ]
            ],
        ]
    ],
];
