<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2019 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.1/1829459
 */

namespace Events\Listener;

use Interop\Container\ContainerInterface;
use Zend\ServiceManager\Factory\FactoryInterface;

class ListenerFactory implements FactoryInterface
{
    const EVENT_LISTENER_CONFIG = 'event_listener_config';
    const PRIORITY              = 'priority';
    const CALLBACK              = 'callback';
    const MANAGER_CONTEXT       = 'context';
    const WORKER_SHUTDOWN       = 'worker.shutdown';
    const WORKER_LOOP           = 'worker.loop';
    const WORKER_STARTUP        = 'worker.startup';
    const CLEANUP_ATTACHMENTS   = 'task.cleanup.attachment';
    const CLEANUP_ARCHIVE       = 'task.cleanup.archive';
    const COMMIT                = 'task.commit';
    const SHELVE                = 'task.shelve';
    const SHELVE_DELETE         = 'task.shelvedel';
    const CHANGE_SAVED          = 'task.changesaved';
    const CHANGE_SAVE           = 'task.changesave';
    const COMMENT               = 'task.comment';
    const COMMENT_BATCH         = 'task.comment.batch';
    const COMMENT_SEND_DELAY    = 'task.commentSendDelay';
    const GROUP                 = 'task.group';
    const GROUP_DELETE          = 'task.groupdel';
    const JOB                   = 'task.job';
    const CHANGE                = 'task.change';
    const REVIEW                = 'task.review';
    const PING                  = 'task.ping';
    const USER                  = 'task.user';
    const USER_DELETE           = 'task.userdel';
    const MAIL                  = 'task.mail';
    const ALL                   = '*';

    // Priority for events
    const DEFAULT_PRIORITY     = 1;
    const HANDLE_ACTIVITY      = -200;
    const HANDLE_MAIL_PRIORITY = -300;


    /**
     * @inheritDoc
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $globalConfiguration = $container->get('config');
        $configuration       = [];

        if (array_key_exists(self::EVENT_LISTENER_CONFIG, $globalConfiguration)) {
            $configuration = $globalConfiguration[self::EVENT_LISTENER_CONFIG];
        }

        return new $requestedName($container, $this->recursiveFind($configuration, $requestedName));
    }

    public function recursiveFind(array $haystack, $needle)
    {
        $events    = [];
        $iterator  = new \RecursiveArrayIterator($haystack);
        $recursive = new \RecursiveIteratorIterator(
            $iterator,
            \RecursiveIteratorIterator::SELF_FIRST
        );
        foreach ($recursive as $key => $value) {
            if ($key === $needle) {
                foreach ($value as $callback) {
                    $events[$recursive->getSubIterator($recursive->getDepth() - 1)->key()][] = $callback;
                }
            }
        }
        return $events;
    }
}
