<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2019 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.1/1829459
 */

return array(
    'groups' => array(
        'edit_name_admin_only' => false,    // if enabled only admin users can edit group name
        'super_only'           => false,    // if enabled only super users can view groups pages
    ),
    'router' => array(
        'routes' => array(
            'group' => array(
                'type' => 'Application\Router\Regex',
                'options' => array(
                    'regex'    => '/group(s?)/(?P<group>.+[^/])(/?)',
                    'spec'     => '/groups/%group%',
                    'defaults' => array(
                        'controller' => Groups\Controller\IndexController::class,
                        'action'     => 'group'
                    ),
                ),
            ),
            'add-group' => array(
                'type' => 'Zend\Router\Http\Segment',
                'options' => array(
                    'route'    => '/group[s]/add[/]',
                    'defaults' => array(
                        'controller' => Groups\Controller\IndexController::class,
                        'action'     => 'add',
                    ),
                ),
            ),
            'edit-group' => array(
                'type' => 'Application\Router\Regex',
                'options' => array(
                    'regex'    => '/group(s?)/(?P<group>.+[^/])(/?)/settings(/?)',
                    'spec'     => '/groups/%group%/settings/',
                    'defaults' => array(
                        'controller' => Groups\Controller\IndexController::class,
                        'action'     => 'edit'
                    ),
                ),
            ),
            'edit-notifications' => array(
                'type' => 'Application\Router\Regex',
                'options' => array(
                    'regex'    => '/group(s?)/(?P<group>.+[^/])(/?)/notifications(/?)',
                    'spec'     => '/groups/%group%/notifications/',
                    'defaults' => array(
                        'controller' => Groups\Controller\IndexController::class,
                        'action'     => 'notifications'
                    ),
                ),
            ),
            'delete-group' => array(
                'type' => 'Application\Router\Regex',
                'options' => array(
                    'regex'    => '/group(s?)/delete/(?P<group>.+[^/])(/?)',
                    'spec'     => '/groups/delete/%group%',
                    'defaults' => array(
                        'controller' => Groups\Controller\IndexController::class,
                        'action'     => 'delete'
                    ),
                ),
            ),
            'group-reviews' => array(
                'type' => 'Zend\Router\Http\Segment',
                'options' => array(
                    'route'    => '/group[s][/:group]/reviews[/]',
                    'defaults' => array(
                        'controller' => Groups\Controller\IndexController::class,
                        'action'     => 'reviews'
                    ),
                ),
            ),
            'groups' => array(
                'type' => 'Zend\Router\Http\Segment',
                'options' => array(
                    'route'    => '/groups[/]',
                    'defaults' => array(
                        'controller' => Groups\Controller\IndexController::class,
                        'action'     => 'groups'
                    ),
                ),
            ),
        ),
    ),
    'controllers' => array(
        'factories' => array(
            Groups\Controller\IndexController::class => Application\Controller\IndexControllerFactory::class
        ),
    ),
    'view_manager' => array(
        'template_path_stack'   => array(
            __DIR__ . '/../view',
        ),
    ),
    'view_helpers' => array(
        'factories' => array_fill_keys(
            [
                Application\View\Helper\ViewHelperFactory::GROUP_TOOLBAR,
                Application\View\Helper\ViewHelperFactory::GROUP_SIDEBAR,
                Application\View\Helper\ViewHelperFactory::GROUP_AVATAR,
                Application\View\Helper\ViewHelperFactory::GROUP_AVATARS,
                Application\View\Helper\ViewHelperFactory::GROUP_NOT_SETTINGS
            ],
            Application\View\Helper\ViewHelperFactory::class
        )
    ),
);
